﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.Net;
using System.Xml.Linq;
using System.IO;
using System.Web.UI;

namespace WebRole.Controllers
{
    public class Metrics : TableServiceEntity
    {
        public string TotalRequests { get; set; }
        public Metrics() { }
    }

    public class HomeController : Controller
    {
        public ActionResult Index()
        {
            if (Request.Cookies["AccountName"] == null)
            {
                return RedirectToAction("Login");
            }
            return View();
        }

        public ActionResult Login()
        {
            return View();
        }

        [OutputCache(Location = OutputCacheLocation.None)]
        public ActionResult Load(string service)
        {
            var creds = new StorageCredentialsAccountAndKey(Request.Cookies["AccountName"].Value, Request.Cookies["AccountKey"].Value);
            var req = (HttpWebRequest)WebRequest.Create(string.Format("http://{0}.{1}.core.windows.net/?restype=service&comp=properties", creds.AccountName, service));
            req.Headers["x-ms-version"] = "2009-09-19";
            if (service == "table")
            {
                creds.SignRequestLite(req);
            }
            else
            {
                creds.SignRequest(req);
            }

            return Content(XDocument.Parse(new StreamReader(req.GetResponse().GetResponseStream()).ReadToEnd()).ToString());
        }

        [HttpPost]
        [ValidateInput(false)]
        public ActionResult Save(string service)
        {
            var creds = new StorageCredentialsAccountAndKey(Request.Cookies["AccountName"].Value, Request.Cookies["AccountKey"].Value);
            var req = (HttpWebRequest)WebRequest.Create(string.Format("http://{0}.{1}.core.windows.net/?restype=service&comp=properties", creds.AccountName, service));
            req.Method = "PUT";
            req.Headers["x-ms-version"] = "2009-09-19";
            req.ContentLength = Request.InputStream.Length;
            if (service == "table")
            {
                creds.SignRequestLite(req);
            }
            else
            {
                creds.SignRequest(req);
            }

            using (var stream = req.GetRequestStream())
            {
                Request.InputStream.CopyTo(stream);
                stream.Close();
            }

            try
            {
                req.GetResponse();
                return new EmptyResult();
            }
            catch (WebException e)
            {
                Response.StatusCode = 500;
                Response.TrySkipIisCustomErrors = true;
                return Content(new StreamReader(e.Response.GetResponseStream()).ReadToEnd());
            }
        }

        [HttpPost]
        public ActionResult CompleteLogin(string AccountName, string AccountKey)
        {
            Response.Cookies["AccountName"].Value = AccountName;
            Response.Cookies["AccountKey"].Value = AccountKey;
            return RedirectToAction("Index");
        }

        public ActionResult Logout()
        {
            Response.Cookies.Remove("AccountName");
            Response.Cookies.Remove("AccountKey");
            return RedirectToAction("Login");
        }

        public ActionResult Graph()
        {
            if (Request.Cookies["AccountName"] == null)
            {
                return RedirectToAction("Login");
            }
            IEnumerable<Metrics> metrics = null;
            try
            {
                var ctx = new CloudStorageAccount(new StorageCredentialsAccountAndKey(Request.Cookies["AccountName"].Value, Request.Cookies["AccountKey"].Value), false).CreateCloudTableClient().GetDataServiceContext();
                metrics = ctx.CreateQuery<Metrics>("$MetricsTransactionsBlob").Where(m => string.Compare(m.PartitionKey, (DateTime.UtcNow - TimeSpan.FromDays(1)).ToString("yyyyMMddTHH00")) > 0 && m.RowKey == "user;GetBlob").ToList();
            }
            catch { } // just pass a null to the view in case of errors... the view handles this by displaying a generic error message
            return View(metrics);
        }
    }
}
